//---------------------------------------------------------------------------
#include "MigrationFC.h"
//---------------------------------------------------------------------------




////////////////////////////////////////////////////////////////////////////////
TMigrationFC::TMigrationFC(/*TFragment* fragment*/)
{
	// create new controller for assessing migration candidacy
	C1 = new TFuzzyController();

	// add input and output variables
  	C1->clear_input_variables();
	C1->add_input_variable(new TVariable("RemoteDemand",     "", 00.0,   100.0));
	C1->add_input_variable(new TVariable("LocalDemand",      "", 00.0,   100.0));
	C1->output_variable =  new TVariable("RemoteAttraction", "", 00.0,   100.0);

                    // bias variable, <50 means fast becomes high
    int centre;     // bias variable, 50 means linearly becomes high
                    // bias variable, >50 means slow becomes high


	// add fuzzysets to the input and output variables
    centre  = 40;   // input 1
  	C1->input_variables[0]->clear_fuzzysets();
    C1->input_variables[0]->add_fuzzyset(new TFuzzySet(C1->input_variables[0], "Low", 	    S_CURVE  , DOWN , 0.00*centre, -0.0,        0.50*centre,         -0.0,                1.00*centre));
    C1->input_variables[0]->add_fuzzyset(new TFuzzySet(C1->input_variables[0], "Medium", 	PI_CURVE , UP   , 0.00*centre, 0.50*centre, 1.00*centre,         (100.0-0.50*centre), 100.0      ));
    C1->input_variables[0]->add_fuzzyset(new TFuzzySet(C1->input_variables[0], "High",   	S_CURVE  , UP   , 1.00*centre, -0.0,        (100.0-0.50*centre), -0.0,                100.0      ));
    centre  = 60;   // input 2
  	C1->input_variables[1]->clear_fuzzysets();
    C1->input_variables[1]->add_fuzzyset(new TFuzzySet(C1->input_variables[1], "Low", 	    S_CURVE  , DOWN , 0.00*centre, -0.0,        0.50*centre,         -0.0,                1.00*centre));
    C1->input_variables[1]->add_fuzzyset(new TFuzzySet(C1->input_variables[1], "Medium", 	PI_CURVE , UP   , 0.00*centre, 0.50*centre, 1.00*centre,         (100.0-0.50*centre), 100.0      ));
    C1->input_variables[1]->add_fuzzyset(new TFuzzySet(C1->input_variables[1], "High",   	S_CURVE  , UP   , 1.00*centre, -0.0,        (100.0-0.50*centre), -0.0,                100.0      ));
    centre  = 50;   // output
  	C1->output_variable->clear_fuzzysets();
    C1->output_variable->add_fuzzyset(new TFuzzySet(C1->output_variable, "Low", 	S_CURVE  , DOWN , 0.00*centre, -0.0,        0.50*centre,         -0.0,                1.00*centre));
    C1->output_variable->add_fuzzyset(new TFuzzySet(C1->output_variable, "Medium", 	PI_CURVE , UP   , 0.00*centre, 0.50*centre, 1.00*centre,         (100.0-0.50*centre), 100.0      ));
    C1->output_variable->add_fuzzyset(new TFuzzySet(C1->output_variable, "High",   	S_CURVE  , UP   , 1.00*centre, -0.0,        (100.0-0.50*centre), -0.0,                100.0      ));


	// create empty rules    (conditional, alpha,  weight)
    C1->clear_rules();
    C1->add_rule(new TFuzzyRule(true,         0.00,   1.0));
    C1->add_rule(new TFuzzyRule(true,         0.00,   1.0));
    C1->add_rule(new TFuzzyRule(true,         0.00,   1.0));
    C1->add_rule(new TFuzzyRule(true,         0.00,   1.0));
    C1->add_rule(new TFuzzyRule(true,         0.00,   1.0));
    C1->add_rule(new TFuzzyRule(true,         0.00,   1.0));
    C1->add_rule(new TFuzzyRule(true,         0.00,   1.0));
    C1->add_rule(new TFuzzyRule(true,         0.00,   1.0));
    C1->add_rule(new TFuzzyRule(true,         0.00,   1.0));


	// setup rules              remote_demand        local_demand         candidacy
    //----------------------------------------------------------------------------------
    //
    if (C1->rules[0] != NULL)
    {
    C1->rules[0]->predicate->clear_premises();
    C1->rules[0]->predicate->clear_operators();
    C1->rules[0]->predicate->add_premise(new TPremise(C1->input_variables[0]->fuzzysets[0]));
    C1->rules[0]->predicate->add_operator(0);
    C1->rules[0]->predicate->add_premise(new TPremise(C1->input_variables[1]->fuzzysets[0]));
    C1->rules[0]->consequent->set_premise(new TPremise(C1->output_variable->fuzzysets[0]));
    }
    if (C1->rules[1] != NULL)
    {
    C1->rules[1]->predicate->clear_premises();
    C1->rules[1]->predicate->clear_operators();
    C1->rules[1]->predicate->add_premise(new TPremise(C1->input_variables[0]->fuzzysets[0]));
    C1->rules[1]->predicate->add_operator(0);
    C1->rules[1]->predicate->add_premise(new TPremise(C1->input_variables[1]->fuzzysets[1]));
    C1->rules[1]->consequent->set_premise(new TPremise(C1->output_variable->fuzzysets[0]));
    }
    if (C1->rules[2] != NULL)
    {
    C1->rules[2]->predicate->clear_premises();
    C1->rules[2]->predicate->clear_operators();
    C1->rules[2]->predicate->add_premise(new TPremise(C1->input_variables[0]->fuzzysets[0]));
    C1->rules[2]->predicate->add_operator(0);
    C1->rules[2]->predicate->add_premise(new TPremise(C1->input_variables[1]->fuzzysets[2]));
    C1->rules[2]->consequent->set_premise(new TPremise(C1->output_variable->fuzzysets[0]));
    }
    if (C1->rules[3] != NULL)
    {
    C1->rules[3]->predicate->clear_premises();
    C1->rules[3]->predicate->clear_operators();
    C1->rules[3]->predicate->add_premise(new TPremise(C1->input_variables[0]->fuzzysets[1]));
    C1->rules[3]->predicate->add_operator(0);
    C1->rules[3]->predicate->add_premise(new TPremise(C1->input_variables[1]->fuzzysets[0]));
    C1->rules[3]->consequent->set_premise(new TPremise(C1->output_variable->fuzzysets[1]));
    }
    if (C1->rules[4] != NULL)
    {
    C1->rules[4]->predicate->clear_premises();
    C1->rules[4]->predicate->clear_operators();
    C1->rules[4]->predicate->add_premise(new TPremise(C1->input_variables[0]->fuzzysets[1]));
    C1->rules[4]->predicate->add_operator(0);
    C1->rules[4]->predicate->add_premise(new TPremise(C1->input_variables[1]->fuzzysets[1]));
    C1->rules[4]->consequent->set_premise(new TPremise(C1->output_variable->fuzzysets[0]));
    }
    if (C1->rules[5] != NULL)
    {
    C1->rules[5]->predicate->clear_premises();
    C1->rules[5]->predicate->clear_operators();
    C1->rules[5]->predicate->add_premise(new TPremise(C1->input_variables[0]->fuzzysets[1]));
    C1->rules[5]->predicate->add_operator(0);
    C1->rules[5]->predicate->add_premise(new TPremise(C1->input_variables[1]->fuzzysets[2]));
    C1->rules[5]->consequent->set_premise(new TPremise(C1->output_variable->fuzzysets[0]));
    }
    if (C1->rules[6] != NULL)
    {
    C1->rules[6]->predicate->clear_premises();
    C1->rules[6]->predicate->clear_operators();
    C1->rules[6]->predicate->add_premise(new TPremise(C1->input_variables[0]->fuzzysets[2]));
    C1->rules[6]->predicate->add_operator(0);
    C1->rules[6]->predicate->add_premise(new TPremise(C1->input_variables[1]->fuzzysets[0]));
    C1->rules[6]->consequent->set_premise(new TPremise(C1->output_variable->fuzzysets[2]));
    }
    if (C1->rules[7] != NULL)
    {
    C1->rules[7]->predicate->clear_premises();
    C1->rules[7]->predicate->clear_operators();
    C1->rules[7]->predicate->add_premise(new TPremise(C1->input_variables[0]->fuzzysets[2]));
    C1->rules[7]->predicate->add_operator(0);
    C1->rules[7]->predicate->add_premise(new TPremise(C1->input_variables[1]->fuzzysets[1]));
    C1->rules[7]->consequent->set_premise(new TPremise(C1->output_variable->fuzzysets[1]));
    }
    if (C1->rules[8] != NULL)
    {
    C1->rules[8]->predicate->clear_premises();
    C1->rules[8]->predicate->clear_operators();
    C1->rules[8]->predicate->add_premise(new TPremise(C1->input_variables[0]->fuzzysets[2]));
    C1->rules[8]->predicate->add_operator(0);
    C1->rules[8]->predicate->add_premise(new TPremise(C1->input_variables[1]->fuzzysets[2]));
    C1->rules[8]->consequent->set_premise(new TPremise(C1->output_variable->fuzzysets[0]));
    }


//    Choose best 16 from 256 as candidates


	// create new controller for assessing migration goodness
	C2 = new TFuzzyController();

	// add input and output variables
  	C2->clear_input_variables();
	C2->add_input_variable(new TVariable("LocalLoad",      "", 00.0,   100.0));
	C2->add_input_variable(new TVariable("RemoteLoad",     "", 00.0,   100.0));
	C2->output_variable =  new TVariable("LocalRepulsion", "", 00.0,   100.0);

	// add fuzzysets to the input and output variables
    centre  = 60;   // input 1
  	C2->input_variables[0]->clear_fuzzysets();
    C2->input_variables[0]->add_fuzzyset(new TFuzzySet(C2->input_variables[0], "Low", 	    S_CURVE  , DOWN , 0.00*centre, -0.0,        0.50*centre,         -0.0,                1.00*centre));
    C2->input_variables[0]->add_fuzzyset(new TFuzzySet(C2->input_variables[0], "Medium", 	PI_CURVE , UP   , 0.00*centre, 0.50*centre, 1.00*centre,         (100.0-0.50*centre), 100.0      ));
    C2->input_variables[0]->add_fuzzyset(new TFuzzySet(C2->input_variables[0], "High",   	S_CURVE  , UP   , 1.00*centre, -0.0,        (100.0-0.50*centre), -0.0,                100.0      ));
    centre  = 40;   // input 2
  	C2->input_variables[1]->clear_fuzzysets();
    C2->input_variables[1]->add_fuzzyset(new TFuzzySet(C2->input_variables[1], "Low", 	    S_CURVE  , DOWN , 0.00*centre, -0.0,        0.50*centre,         -0.0,                1.00*centre));
    C2->input_variables[1]->add_fuzzyset(new TFuzzySet(C2->input_variables[1], "Medium", 	PI_CURVE , UP   , 0.00*centre, 0.50*centre, 1.00*centre,         (100.0-0.50*centre), 100.0      ));
    C2->input_variables[1]->add_fuzzyset(new TFuzzySet(C2->input_variables[1], "High",   	S_CURVE  , UP   , 1.00*centre, -0.0,        (100.0-0.50*centre), -0.0,                100.0      ));
    centre  = 50;   // output
  	C2->output_variable->clear_fuzzysets();
    C2->output_variable->add_fuzzyset(new TFuzzySet(C2->output_variable, "Low", 	S_CURVE  , DOWN , 0.00*centre, -0.0,        0.50*centre,         -0.0,                1.00*centre));
    C2->output_variable->add_fuzzyset(new TFuzzySet(C2->output_variable, "Medium", 	PI_CURVE , UP   , 0.00*centre, 0.50*centre, 1.00*centre,         (100.0-0.50*centre), 100.0      ));
    C2->output_variable->add_fuzzyset(new TFuzzySet(C2->output_variable, "High",   	S_CURVE  , UP   , 1.00*centre, -0.0,        (100.0-0.50*centre), -0.0,                100.0      ));


	// create empty rules    (conditional, alpha,  weight)
    C2->clear_rules();
    C2->add_rule(new TFuzzyRule(true,         0.00,   1.0));
    C2->add_rule(new TFuzzyRule(true,         0.00,   1.0));
    C2->add_rule(new TFuzzyRule(true,         0.00,   1.0));
    C2->add_rule(new TFuzzyRule(true,         0.00,   1.0));
    C2->add_rule(new TFuzzyRule(true,         0.00,   1.0));
    C2->add_rule(new TFuzzyRule(true,         0.00,   1.0));
    C2->add_rule(new TFuzzyRule(true,         0.00,   1.0));
    C2->add_rule(new TFuzzyRule(true,         0.00,   1.0));
    C2->add_rule(new TFuzzyRule(true,         0.00,   1.0));


	// setup rules                local_load         remote_load        local_repulsion
    //----------------------------------------------------------------------------------
    //
    if (C2->rules[0] != NULL)
    {
    C2->rules[0]->predicate->clear_premises();
    C2->rules[0]->predicate->clear_operators();
    C2->rules[0]->predicate->add_premise(new TPremise(C2->input_variables[0]->fuzzysets[0]));
    C2->rules[0]->predicate->add_operator(0);
    C2->rules[0]->predicate->add_premise(new TPremise(C2->input_variables[1]->fuzzysets[0]));
    C2->rules[0]->consequent->set_premise(new TPremise(C2->output_variable->fuzzysets[0]));
    }
    if (C2->rules[1] != NULL)
    {
    C2->rules[1]->predicate->clear_premises();
    C2->rules[1]->predicate->clear_operators();
    C2->rules[1]->predicate->add_premise(new TPremise(C2->input_variables[0]->fuzzysets[0]));
    C2->rules[1]->predicate->add_operator(0);
    C2->rules[1]->predicate->add_premise(new TPremise(C2->input_variables[1]->fuzzysets[1]));
    C2->rules[1]->consequent->set_premise(new TPremise(C2->output_variable->fuzzysets[0]));
    }
    if (C2->rules[2] != NULL)
    {
    C2->rules[2]->predicate->clear_premises();
    C2->rules[2]->predicate->clear_operators();
    C2->rules[2]->predicate->add_premise(new TPremise(C2->input_variables[0]->fuzzysets[0]));
    C2->rules[2]->predicate->add_operator(0);
    C2->rules[2]->predicate->add_premise(new TPremise(C2->input_variables[1]->fuzzysets[2]));
    C2->rules[2]->consequent->set_premise(new TPremise(C2->output_variable->fuzzysets[0]));
    }
    if (C2->rules[3] != NULL)
    {
    C2->rules[3]->predicate->clear_premises();
    C2->rules[3]->predicate->clear_operators();
    C2->rules[3]->predicate->add_premise(new TPremise(C2->input_variables[0]->fuzzysets[1]));
    C2->rules[3]->predicate->add_operator(0);
    C2->rules[3]->predicate->add_premise(new TPremise(C2->input_variables[1]->fuzzysets[0]));
    C2->rules[3]->consequent->set_premise(new TPremise(C2->output_variable->fuzzysets[1]));
    }
    if (C2->rules[4] != NULL)
    {
    C2->rules[4]->predicate->clear_premises();
    C2->rules[4]->predicate->clear_operators();
    C2->rules[4]->predicate->add_premise(new TPremise(C2->input_variables[0]->fuzzysets[1]));
    C2->rules[4]->predicate->add_operator(0);
    C2->rules[4]->predicate->add_premise(new TPremise(C2->input_variables[1]->fuzzysets[1]));
    C2->rules[4]->consequent->set_premise(new TPremise(C2->output_variable->fuzzysets[0]));
    }
    if (C2->rules[5] != NULL)
    {
    C2->rules[5]->predicate->clear_premises();
    C2->rules[5]->predicate->clear_operators();
    C2->rules[5]->predicate->add_premise(new TPremise(C2->input_variables[0]->fuzzysets[1]));
    C2->rules[5]->predicate->add_operator(0);
    C2->rules[5]->predicate->add_premise(new TPremise(C2->input_variables[1]->fuzzysets[2]));
    C2->rules[5]->consequent->set_premise(new TPremise(C2->output_variable->fuzzysets[0]));
    }
    if (C2->rules[6] != NULL)
    {
    C2->rules[6]->predicate->clear_premises();
    C2->rules[6]->predicate->clear_operators();
    C2->rules[6]->predicate->add_premise(new TPremise(C2->input_variables[0]->fuzzysets[2]));
    C2->rules[6]->predicate->add_operator(0);
    C2->rules[6]->predicate->add_premise(new TPremise(C2->input_variables[1]->fuzzysets[0]));
    C2->rules[6]->consequent->set_premise(new TPremise(C2->output_variable->fuzzysets[2]));
    }
    if (C2->rules[7] != NULL)
    {
    C2->rules[7]->predicate->clear_premises();
    C2->rules[7]->predicate->clear_operators();
    C2->rules[7]->predicate->add_premise(new TPremise(C2->input_variables[0]->fuzzysets[2]));
    C2->rules[7]->predicate->add_operator(0);
    C2->rules[7]->predicate->add_premise(new TPremise(C2->input_variables[1]->fuzzysets[1]));
    C2->rules[7]->consequent->set_premise(new TPremise(C2->output_variable->fuzzysets[1]));
    }
    if (C2->rules[8] != NULL)
    {
    C2->rules[8]->predicate->clear_premises();
    C2->rules[8]->predicate->clear_operators();
    C2->rules[8]->predicate->add_premise(new TPremise(C2->input_variables[0]->fuzzysets[2]));
    C2->rules[8]->predicate->add_operator(0);
    C2->rules[8]->predicate->add_premise(new TPremise(C2->input_variables[1]->fuzzysets[2]));
    C2->rules[8]->consequent->set_premise(new TPremise(C2->output_variable->fuzzysets[0]));
    }


//    Apply on the 16 candidates

	// create new controller for assessing migration goodness
	C3 = new TFuzzyController();

	// add input and output variables
  	C3->clear_input_variables();
	C3->add_input_variable(C1->output_variable);
	C3->add_input_variable(C2->output_variable);
	C3->output_variable =  new TVariable("Migration",        "", 00.0,   100.0);

	// add fuzzysets to the output variable
    centre  = 50;   // output
  	C3->output_variable->clear_fuzzysets();
    C3->output_variable->add_fuzzyset(new TFuzzySet(C3->output_variable, "VeryLow", S_CURVE  , DOWN , 0.00*centre,          -0.0,                 0.25*centre,          -0.0,                 0.50*centre         ));
    C3->output_variable->add_fuzzyset(new TFuzzySet(C3->output_variable, "Low", 	PI_CURVE , UP   , 0.00*centre,          0.25*centre,          0.50*centre,          0.75*centre,          1.00*centre         ));
    C3->output_variable->add_fuzzyset(new TFuzzySet(C3->output_variable, "Medium", 	PI_CURVE , UP   , 0.50*centre,          0.75*centre,          1.00*centre,          (100.00-0.70*(100-centre)), (100.00-0.50*(100-centre))));
    C3->output_variable->add_fuzzyset(new TFuzzySet(C3->output_variable, "High", 	PI_CURVE , UP   , 1.00*centre,          (100.00-0.75*(100-centre)), (100.00-0.50*(100-centre)), (100.00-0.25*(100-centre)), 100.00  ));
    C3->output_variable->add_fuzzyset(new TFuzzySet(C3->output_variable, "VeryHigh",S_CURVE  , UP   , (100.00-0.50*(100-centre)), -0.0,                 (100.00-0.25*(100-centre)), -0.0,                 100.00        ));

	// create empty rules    (conditional, alpha,  weight)
    C3->clear_rules();
    C3->add_rule(new TFuzzyRule(true,         0.05,   1.0));
    C3->add_rule(new TFuzzyRule(true,         0.05,   1.0));
    C3->add_rule(new TFuzzyRule(true,         0.05,   1.0));
    C3->add_rule(new TFuzzyRule(true,         0.05,   1.0));
    C3->add_rule(new TFuzzyRule(true,         0.05,   1.0));
    C3->add_rule(new TFuzzyRule(true,         0.05,   1.0));
    C3->add_rule(new TFuzzyRule(true,         0.05,   1.0));
    C3->add_rule(new TFuzzyRule(true,         0.05,   1.0));
    C3->add_rule(new TFuzzyRule(true,         0.05,   1.0));


	// setup rules             remote_attraction     local_repulsion       migration
    //----------------------------------------------------------------------------------
    //
    if (C3->rules[0] != NULL)
    {
    C3->rules[0]->predicate->clear_premises();
    C3->rules[0]->predicate->clear_operators();
    C3->rules[0]->predicate->add_premise(new TPremise(C3->input_variables[0]->fuzzysets[0]));
    C3->rules[0]->predicate->add_operator(0);
    C3->rules[0]->predicate->add_premise(new TPremise(C3->input_variables[1]->fuzzysets[0]));
    C3->rules[0]->consequent->set_premise(new TPremise(C3->output_variable->fuzzysets[0]));
    }
    if (C3->rules[1] != NULL)
    {
    C3->rules[1]->predicate->clear_premises();
    C3->rules[1]->predicate->clear_operators();
    C3->rules[1]->predicate->add_premise(new TPremise(C3->input_variables[0]->fuzzysets[0]));
    C3->rules[1]->predicate->add_operator(0);
    C3->rules[1]->predicate->add_premise(new TPremise(C3->input_variables[1]->fuzzysets[1]));
    C3->rules[1]->consequent->set_premise(new TPremise(C3->output_variable->fuzzysets[1]));
    }
    if (C3->rules[2] != NULL)
    {
    C3->rules[2]->predicate->clear_premises();
    C3->rules[2]->predicate->clear_operators();
    C3->rules[2]->predicate->add_premise(new TPremise(C3->input_variables[0]->fuzzysets[0]));
    C3->rules[2]->predicate->add_operator(0);
    C3->rules[2]->predicate->add_premise(new TPremise(C3->input_variables[1]->fuzzysets[2]));
    C3->rules[2]->consequent->set_premise(new TPremise(C3->output_variable->fuzzysets[2]));
    }

    if (C3->rules[3] != NULL)
    {
    C3->rules[3]->predicate->clear_premises();
    C3->rules[3]->predicate->clear_operators();
    C3->rules[3]->predicate->add_premise(new TPremise(C3->input_variables[0]->fuzzysets[1]));
    C3->rules[3]->predicate->add_operator(0);
    C3->rules[3]->predicate->add_premise(new TPremise(C3->input_variables[1]->fuzzysets[0]));
    C3->rules[3]->consequent->set_premise(new TPremise(C3->output_variable->fuzzysets[1]));
    }
    if (C3->rules[4] != NULL)
    {
    C3->rules[4]->predicate->clear_premises();
    C3->rules[4]->predicate->clear_operators();
    C3->rules[4]->predicate->add_premise(new TPremise(C3->input_variables[0]->fuzzysets[1]));
    C3->rules[4]->predicate->add_operator(0);
    C3->rules[4]->predicate->add_premise(new TPremise(C3->input_variables[1]->fuzzysets[1]));
    C3->rules[4]->consequent->set_premise(new TPremise(C3->output_variable->fuzzysets[2]));
    }
    if (C3->rules[5] != NULL)
    {
    C3->rules[5]->predicate->clear_premises();
    C3->rules[5]->predicate->clear_operators();
    C3->rules[5]->predicate->add_premise(new TPremise(C3->input_variables[0]->fuzzysets[1]));
    C3->rules[5]->predicate->add_operator(0);
    C3->rules[5]->predicate->add_premise(new TPremise(C3->input_variables[1]->fuzzysets[2]));
    C3->rules[5]->consequent->set_premise(new TPremise(C3->output_variable->fuzzysets[3]));
    }

    if (C3->rules[6] != NULL)
    {
    C3->rules[6]->predicate->clear_premises();
    C3->rules[6]->predicate->clear_operators();
    C3->rules[6]->predicate->add_premise(new TPremise(C3->input_variables[0]->fuzzysets[2]));
    C3->rules[6]->predicate->add_operator(0);
    C3->rules[6]->predicate->add_premise(new TPremise(C3->input_variables[1]->fuzzysets[0]));
    C3->rules[6]->consequent->set_premise(new TPremise(C3->output_variable->fuzzysets[2]));
    }
    if (C3->rules[7] != NULL)
    {
    C3->rules[7]->predicate->clear_premises();
    C3->rules[7]->predicate->clear_operators();
    C3->rules[7]->predicate->add_premise(new TPremise(C3->input_variables[0]->fuzzysets[2]));
    C3->rules[7]->predicate->add_operator(0);
    C3->rules[7]->predicate->add_premise(new TPremise(C3->input_variables[1]->fuzzysets[1]));
    C3->rules[7]->consequent->set_premise(new TPremise(C3->output_variable->fuzzysets[3]));
    }
    if (C3->rules[8] != NULL)
    {
    C3->rules[8]->predicate->clear_premises();
    C3->rules[8]->predicate->clear_operators();
    C3->rules[8]->predicate->add_premise(new TPremise(C3->input_variables[0]->fuzzysets[2]));
    C3->rules[8]->predicate->add_operator(0);
    C3->rules[8]->predicate->add_premise(new TPremise(C3->input_variables[1]->fuzzysets[2]));
    C3->rules[8]->consequent->set_premise(new TPremise(C3->output_variable->fuzzysets[4]));
    }

    for (int i=0; i<MAX_DEMANDING_NODES; i++)
    {
        remote_attractionsets[i] = NULL;
    }
    for (int i=0; i<MAX_CANDIDATE_NODES; i++)
    {
        local_repulsionsets[i] = NULL;
    }
    for (int i=0; i<MAX_CANDIDATE_NODES; i++)
    {
        migrationsets[i] = NULL;
    }
}
////////////////////////////////////////////////////////////////////////////////



////////////////////////////////////////////////////////////////////////////////
TMigrationFC::~TMigrationFC()
{
    if (C1 != NULL)	delete	C1;    C1 = NULL;
    if (C2 != NULL)	delete	C2;    C2 = NULL;
    if (C3 != NULL)	delete	C3;    C3 = NULL;
}
////////////////////////////////////////////////////////////////////////////////







// fill demanding_nodes_ids[MAX_DEMANDING_NODES]
void TMigrationFC::IdentifyDemandingNodes()  // using Fragment->DemandHistoryTable[i]
{
// temporary
    for (int i=0; i<MAX_DEMANDING_NODES; i++)
    {
        demanding_nodes_ids[i] = i;
    }
}




// average info from fragment's statistics in last T (+ forcast)
void TMigrationFC::GetLocalDemand() // using Fragment->DemandHistoryTable[i]
{
// temporary
    local_demand = random(100);
}


// for each demanding node in demanding_nodes_ids[MAX_DEMANDING_NODES]
// fill remote_loads[index]
void TMigrationFC::GetRemoteDemand(int index)  // using Fragment->DemandHistoryTable[i]
{
// temporary
    for (int i=0; i<MAX_DEMANDING_NODES; i++)
    {
        remote_demands[i] = random(100);
    }
}


// for each demanding node, run C1 to get its attraction force and
// fill remote_attractions[MAX_DEMANDING_NODES] with the results
int TMigrationFC::RunC1(int index, double input1, double input2,
int AggregationMethod, int CorrelationMethod, int AlphaCutType, int CompositionMethod, int DefuzzificationMethod)
{
    // apply new inputs
	C1->rules[0]->predicate->premises[0]->set_scalar(input1);
	C1->rules[0]->predicate->premises[1]->set_scalar(input2);
	C1->rules[1]->predicate->premises[0]->set_scalar(input1);
	C1->rules[1]->predicate->premises[1]->set_scalar(input2);
	C1->rules[2]->predicate->premises[0]->set_scalar(input1);
	C1->rules[2]->predicate->premises[1]->set_scalar(input2);
	C1->rules[3]->predicate->premises[0]->set_scalar(input1);
	C1->rules[3]->predicate->premises[1]->set_scalar(input2);
	C1->rules[4]->predicate->premises[0]->set_scalar(input1);
	C1->rules[4]->predicate->premises[1]->set_scalar(input2);
	C1->rules[5]->predicate->premises[0]->set_scalar(input1);
	C1->rules[5]->predicate->premises[1]->set_scalar(input2);
	C1->rules[6]->predicate->premises[0]->set_scalar(input1);
	C1->rules[6]->predicate->premises[1]->set_scalar(input2);
	C1->rules[7]->predicate->premises[0]->set_scalar(input1);
	C1->rules[7]->predicate->premises[1]->set_scalar(input2);
	C1->rules[8]->predicate->premises[0]->set_scalar(input1);
	C1->rules[8]->predicate->premises[1]->set_scalar(input2);

    // run the controller for the new inputs
	remote_attractionsets[index]        = C1->GenerateOutputSet(AggregationMethod, CorrelationMethod, AlphaCutType, CompositionMethod);
	return remote_attractions[index]    = C1->GenerateOutput   (AggregationMethod, CorrelationMethod, AlphaCutType, CompositionMethod, DefuzzificationMethod);
}









// fill (partially) candidate_nodes_ids[MAX_CANDIDATE_NODES] with ids of
// first nodes with attraction force > MEDIUM, the rest fill with INVALID_ID
void TMigrationFC::ChooseCandidates()
{
    for (int i=0; i<MAX_CANDIDATE_NODES; i++)
    {
        remote_demands[i] = i;
    }
}









// fill local_load
void TMigrationFC::GetLocalLoad()  //using Fragment->Node->NodeMonitor->Others[i]
{
// temporary
    local_load = random(100);
}


// for each chosen candidate node
// fill remote_loads[index]
void TMigrationFC::GetRemoteLoad(int index)  //using Fragment->Node->NodeMonitor->Others[i]
{
// temporary
    for (int i=0; i<MAX_CANDIDATE_NODES; i++)
    {
        remote_loads[i] = random(100);
    }
}


// for each chosen candidate node
// run C2 to get the local_repulsion force relative to its load
// fill local_repulsions[MAX_CANDIDATE_NODES] with the results
int TMigrationFC::RunC2(int index, double input1, double input2,
int AggregationMethod, int CorrelationMethod, int AlphaCutType, int CompositionMethod, int DefuzzificationMethod)
{
    // apply new inputs
	C2->rules[0]->predicate->premises[0]->set_scalar(input1);
	C2->rules[0]->predicate->premises[1]->set_scalar(input2);
	C2->rules[1]->predicate->premises[0]->set_scalar(input1);
	C2->rules[1]->predicate->premises[1]->set_scalar(input2);
	C2->rules[2]->predicate->premises[0]->set_scalar(input1);
	C2->rules[2]->predicate->premises[1]->set_scalar(input2);
	C2->rules[3]->predicate->premises[0]->set_scalar(input1);
	C2->rules[3]->predicate->premises[1]->set_scalar(input2);
	C2->rules[4]->predicate->premises[0]->set_scalar(input1);
	C2->rules[4]->predicate->premises[1]->set_scalar(input2);
	C2->rules[5]->predicate->premises[0]->set_scalar(input1);
	C2->rules[5]->predicate->premises[1]->set_scalar(input2);
	C2->rules[6]->predicate->premises[0]->set_scalar(input1);
	C2->rules[6]->predicate->premises[1]->set_scalar(input2);
	C2->rules[7]->predicate->premises[0]->set_scalar(input1);
	C2->rules[7]->predicate->premises[1]->set_scalar(input2);
	C2->rules[8]->predicate->premises[0]->set_scalar(input1);
	C2->rules[8]->predicate->premises[1]->set_scalar(input2);

    // run the controller for the new inputs
	local_repulsionsets[index]      = C2->GenerateOutputSet(AggregationMethod, CorrelationMethod, AlphaCutType, CompositionMethod);
	return local_repulsions[index]  = C2->GenerateOutput   (AggregationMethod, CorrelationMethod, AlphaCutType, CompositionMethod, DefuzzificationMethod);
}



// for each chosen candidate node, run C3 to get the migration force
// fill migrations[MAX_CANDIDATE_NODES] with the results
int TMigrationFC::RunC3(int index, double input1, double input2,
int AggregationMethod, int CorrelationMethod, int AlphaCutType, int CompositionMethod, int DefuzzificationMethod)
{
    // apply new inputs
	C3->rules[0]->predicate->premises[0]->set_scalar(input1);
	C3->rules[0]->predicate->premises[1]->set_scalar(input2);
	C3->rules[1]->predicate->premises[0]->set_scalar(input1);
	C3->rules[1]->predicate->premises[1]->set_scalar(input2);
	C3->rules[2]->predicate->premises[0]->set_scalar(input1);
	C3->rules[2]->predicate->premises[1]->set_scalar(input2);
	C3->rules[3]->predicate->premises[0]->set_scalar(input1);
	C3->rules[3]->predicate->premises[1]->set_scalar(input2);
	C3->rules[4]->predicate->premises[0]->set_scalar(input1);
	C3->rules[4]->predicate->premises[1]->set_scalar(input2);
	C3->rules[5]->predicate->premises[0]->set_scalar(input1);
	C3->rules[5]->predicate->premises[1]->set_scalar(input2);
	C3->rules[6]->predicate->premises[0]->set_scalar(input1);
	C3->rules[6]->predicate->premises[1]->set_scalar(input2);
	C3->rules[7]->predicate->premises[0]->set_scalar(input1);
	C3->rules[7]->predicate->premises[1]->set_scalar(input2);
	C3->rules[8]->predicate->premises[0]->set_scalar(input1);
	C3->rules[8]->predicate->premises[1]->set_scalar(input2);

    // run the controller for the new inputs
	migrationsets[index]        = C3->GenerateOutputSet(AggregationMethod, CorrelationMethod, AlphaCutType, CompositionMethod);
	return migrations[index]    = C3->GenerateOutput   (AggregationMethod, CorrelationMethod, AlphaCutType, CompositionMethod, DefuzzificationMethod);
}




// fill migration_node_id with the id of the node with the highest
// migrations[MAX_CANDIDATE_NODES] to migrate to.
// or fill migration_node_id with INVALID_ID for no migration
void TMigrationFC::SelectCandidate()
{
//
///    int    migration_node_id;
}






// returns the migration_node_id, if any
int TMigrationFC::Run()
{
    int index = 0;
    // fill demanding_nodes_ids[MAX_DEMANDING_NODES]
    IdentifyDemandingNodes();  // using Fragment->DemandHistoryTable[i]


    // fill local_demand
    GetLocalDemand();  // using Fragment->DemandHistoryTable[i]

    // for each demanding node in demanding_nodes_ids[MAX_DEMANDING_NODES]
    // fill remote_loads[index]
    GetRemoteDemand(index);  // using Fragment->DemandHistoryTable[i]

    // for each demanding node, run C1 to get its attraction force and
	// fill remote_attractions[MAX_DEMANDING_NODES] with the results
	RunC1(index, random(100), random(100), 0, 0, 0, 0, 0);


	// fill (partially) candidate_nodes_ids[MAX_CANDIDATE_NODES] with ids of
    // first nodes with attraction force > MEDIUM, the rest fill with INVALID_ID
	ChooseCandidates();



    // fill local_load
    GetLocalLoad();          //using Fragment->Node->NodeMonitor->Others[i]

    // for each chosen candidate node
    // fill remote_loads[index]
    GetRemoteLoad(index);//using Fragment->Node->NodeMonitor->Others[i]


    // for each chosen candidate node
    // run C2 to get the local_repulsion force relative to its load
	// fill local_repulsions[MAX_CANDIDATE_NODES] with the results
	RunC2(index, random(100), random(100), 0, 0, 0, 0, 0);

    // for each chosen candidate node, run C3 to get the migration force
	// fill migrations[MAX_CANDIDATE_NODES] with the results
	RunC3(index, random(100), random(100), 0, 0, 0, 0, 0);


	// fill migration_node_id with the id of the node with the highest
    // migrations[MAX_CANDIDATE_NODES] to migrate to.
    // or fill migration_node_id with INVALID_ID for no migration
	SelectCandidate();


	return migration_node_id;
}
////////////////////////////////////////////////////////////////////////////////



